<?php

namespace ExactLinks\App\Hooks\Handlers;

use ExactLinks\App\App;
use ExactLinks\Framework\Support\Arr;
use ExactLinks\App\Models\Link;
use ExactLinks\App\Models\LinkAnalytics;
use ExactLinks\App\Hooks\Handlers\Builder;
use ExactLinks\App\Libs\Browser\BrowserDetection;
use ExactLinks\App\Libs\BotDetection\BotDetection;

class FrontendHandler
{
    public function redirectionURL()
    {   
        $userAgent = (isset($_SERVER['HTTP_USER_AGENT'])) ? sanitize_text_field($_SERVER['HTTP_USER_AGENT']) : '';
        // Don't track from Bot
        if ((new BotDetection)->is_bot($userAgent)) {
			die;
		}

        $slug = $this->getCurrentUrl();
        
        $post = (new Link)->isSlug($slug);
        
        if (isset($post)) {
            $this->createAnalytics($post);

            $postId = intval($post->id);

            $types = array('simple', 'box_content', 'choice_links', 'ab_links');
            
            if (in_array($post->type, $types)) {
                wp_redirect($post->target_url, $post->redirect_type);
                exit();
            } elseif ($post->type == 'choice_pages') {
                $this->render($postId); 
            } elseif ($post->type == 'ab_pages') {
                $this->abSplitRedirectionURL($postId);
            }
        }   
    }

    public function getCurrentUrl()
    {
        $currentUrl =  sanitize_url("http" . (($_SERVER['SERVER_PORT'] == 443) ? "s" : "") . "://" . $_SERVER['HTTP_HOST'] . $_SERVER['REQUEST_URI']);
        $actualUrl  = explode(sanitize_text_field($_SERVER['HTTP_HOST']), $currentUrl)[1];
        $slug = substr($actualUrl, 1);

        return $slug;
    }

    public function render($postId)
    {  
        $posts = (new Link)->getLink($postId);
        
        if (!$posts) {
            return;
        }

        echo $this->renderPreview($posts); 
        die;
    }

    public function renderPreview($posts)
    {
       $app = App::getInstance();
        
       $app->addAction('wp_enqueue_scripts', [$this, 'enqueueScripts']);

       return $app->view->make('public.Frontendview', compact('posts'));
    }

    public function enqueueScripts()
    {
        wp_enqueue_style('exactlinks-common', EXACTLINKS_PLUGIN_URL.'assets/public/css/exactlinks-common.css');
    }
    
    /**
     * get Traffic Source
    */ 
    public function getTrafficSource()
    {   
        $trafficSourceName = __('Direct', 'exact-links');
        
        if (isset($_SERVER['HTTP_REFERER'])) {
            $trafficSourceName = sanitize_url($_SERVER['HTTP_REFERER']);
        }

        return $trafficSourceName;
    }

    public function getDetectedDevice()
    {

        $browserDetection =  (new BrowserDetection);

        $userAgent = ''; 

        if (isset($_SERVER, $_SERVER['HTTP_USER_AGENT'])) {
            $userAgent = sanitize_text_field($_SERVER['HTTP_USER_AGENT']);
        }

        $browserName = $browserDetection->getBrowser($userAgent);
        $osName = $browserDetection->getOS($userAgent);
        $devicesName = $browserDetection->getDevice($userAgent);
       
        return apply_filters('exactlinks/get_detected_device', [
            'browserName' => $browserName['browser_name'] ?: __('(not set)', 'exact-links'),
            'osName'      => $osName['os_name'] ?: __('(not set)', 'exact-links'),
            'devicesName' => $devicesName['device_type'] ?: __('(not set)', 'exact-links'),
            'trafficSourceName' => $this->getTrafficSource(),
            'ipAddress' =>  App::getInstance("request")->getIp()
        ]);
    }

    /***
     * Insert Traking data 
    */
    public function createAnalytics($post)
    {  
        $deviceInfo = $this->getDetectedDevice();
        // checking Ip Address
        $isIpAddress = (new LinkAnalytics)->isIpAddress($post->id, $deviceInfo['ipAddress']);
        
        //cookie set slug 
        $this->exlSetCookie($post->slug);

        $data = [
            'link_id'             => intval($post->id),
            'slug'                => sanitize_title($post->slug),
            'os_name'             => sanitize_text_field($deviceInfo['osName']),
            'ip'                  => sanitize_text_field($deviceInfo['ipAddress']),
            'devices_name'        => sanitize_text_field($deviceInfo['devicesName']),
            'browser_name'        => sanitize_text_field($deviceInfo['browserName']),
            'traffic_source_name' => $deviceInfo['trafficSourceName'],
            'date'                => date("Y-m-d H:i:s")
        ];
       
        apply_filters('exactlinks/before_create_analytics', $data);

        // insert Analytic data
        (new LinkAnalytics)->insertGetId($data);
        // insert click data
        $this->createClick($post, $isIpAddress);
    }

    /**
     *  A/B Split Testing Redirection URL & insert data
    */
    public function abSplitRedirectionURL($postId)
    {  
        $abSplitPost = $this->abSplitPriority($postId);
        $id          = intval($abSplitPost['id']);
        $deviceInfo  = $this->getDetectedDevice();
        $abLink = (new Link)->where('id', $id)->first(); 
        
        if (!$abLink) {
            return;
        }

        //cookie set slug 
        $this->exlSetCookie($abLink->slug);

        // checking Ip Address
        $ipAddress = App::getInstance("request")->getIp();
        $isIpAddress = (new LinkAnalytics)->isIpAddress($abLink['id'], $ipAddress);
        
        $data = [
            'link_id'      => $id,
            'os_name'      => sanitize_text_field($deviceInfo['osName']),
            'ip'           => sanitize_text_field($deviceInfo['ipAddress']),
            'devices_name' => sanitize_text_field($deviceInfo['devicesName']),
            'browser_name' => sanitize_text_field($deviceInfo['browserName']),
            'traffic_source_name' => $deviceInfo['trafficSourceName'],
            'date'         => date("Y-m-d H:i:s")
        ];

        apply_filters('exactlinks/before_ab_split_create_analytics', $data);

        // insert ablink Analytic data (link wise)
        (new LinkAnalytics)->insertGetId($data);
        // insert click data
        $this->createClick($abLink, $isIpAddress);
        
        wp_redirect($abLink->target_url, $abLink->redirect_type);
        exit();
    }

    /**
     *  A/B Split Testing Priority
    */
    public function abSplitPriority($postId)
    {
        $posts = (new Link)->getLink($postId);
        $posts =  Arr::get($posts, 'ab_links'); 
        
        if (!$posts) {
            return null;
        }

        $priorities = array();
        
        foreach ($posts as $row) {
            $priorities[] = $row['priority'];
        }

        $count = count($priorities);
        $num =  mt_rand(0, array_sum($priorities));
        
        $i = $n = 0;

        while ($i < $count) {
            $n += $priorities[$i];
            if ($n >= $num) break;
            $i++;
        }

        return $posts[$i] ? $posts[$i] : null;
    }

    /**
     * insert click & unique click data when URL redirection
    */
    public function createClick($post, $isIpAddress)
    {
        if (!$isIpAddress) {
            $uniqueClick = $post->total_unique_click + 1;
        } else {
            $uniqueClick = $post->total_unique_click;
            
            if ($uniqueClick == 0) {
                $uniqueClick = 1;
            }
        }

        $data = array(
            'total_click' => intval($post->total_click + 1),
            'total_unique_click' => intval($uniqueClick)
        );
        
        $createClick = (new Link)->where('id', $post->id)->update($data);
        
        return $createClick;
    }

    /**
     * cookie save when short link click
    */

    public function exlSetCookie($slug)
    {
        //cookie set slug 
        if (defined('WC_VERSION') || defined('EDD_VERSION')) {
            $cookie_name = "exactlinks_slug";
            $cookie_value = sanitize_title($slug);
            $settings = get_option('exactlinks_settings');
            setcookie(
                $cookie_name,
                $cookie_value,
                time() + (86400 * $settings['activeCookies'])
            ); // 86400 = 1 day
        }
    }

    /**
     *  cookie save for woocommerce conversion track
    */
    public function maybeCampaignMeta($order, $data)
    {  
        if (isset($_COOKIE['exactlinks_slug'])) {
            $slug = sanitize_title($_COOKIE['exactlinks_slug']);
            
            if ($slug) {
                $order->update_meta_data('_exactlinks_slug', $slug);
            }
        }
    }

    /**
     *  woocommerce conversion track
    */
    public function maybeRecordPayment($orderId, $order)
    {   
        $slug = $order->get_meta('_exactlinks_slug');
       
        if ($slug) {
            $ipAddress = App::getInstance("request")->getIp();
            $link      = (new Link)->isSlug($slug);
            
            if ($link) {
                $data = array(
                    'link_id'          => intval($link->id),
                    'ip'               => $ipAddress,
                    'slug'             => sanitize_title($link->slug),
                    'conversion_text'  => sanitize_text_field('thankyou_url'),
                    'conversion_amount'=> floatval($order->get_total()),
                    'date'             => date("Y-m-d H:i:s"),
                );

                apply_filters('exactlinks/before_woocommerce_create_analytics', $data);

                (new LinkAnalytics)->insertGetId($data);
                //link conversion rate;
                $conversionRate = array(
                    'conversion_rate'  => intval($link->conversion_rate + 1),
                );
                
                (new Link)->where('id', $link->id)->update($conversionRate);
            }
        }
    }

    /**
     *  cookie save in post meta for EDD conversion track
    */
    public function eddCampaignMeta($payment_id)
    {  
        if (isset($_COOKIE['exactlinks_slug'])) {
            $slug = sanitize_title($_COOKIE['exactlinks_slug']);
          
            if ($slug) {
                update_post_meta($payment_id, '_exactlinks_slug', $slug);
            }
        }
    }

    /**
     *  EDD conversion track
    */
    public function eddCompletePurchase($payment_id, $order) {
       
        $slug = get_post_meta($payment_id, '_exactlinks_slug', true );
            
        if ($slug) {
            $ipAddress = App::getInstance("request")->getIp();
            $link      = (new Link)->isSlug($slug);
            
            if ($link) {
                $data = array(
                    'link_id'          => intval($link->id),
                    'ip'               => $ipAddress,
                    'slug'             => sanitize_title($link->slug),
                    'conversion_text'  => sanitize_text_field('thankyou_url'),
                    'conversion_amount'=> floatval($order->total),
                    'date'             => date("Y-m-d H:i:s"),
                );

                apply_filters('exactlinks/before_edd_create_analytics', $data);

                (new LinkAnalytics)->insertGetId($data);
                //link conversion rate;
                $conversionRate = array(
                    'conversion_rate'  => intval($link->conversion_rate + 1),
                );
                
                (new Link)->where('id', $link->id)->update($conversionRate);
            }
        }
    }


    /**
     * Google Analytics track
    */

    public function googleAnaliticsTrack() 
    {
        $settings = get_option('exactlinks_settings');
        
        if ($settings['googleAnalytics']) {

            $getScript = "window.dataLayer = window.dataLayer || []; 
                        function gtag(){dataLayer.push(arguments);}
                        gtag('js', new Date()); 
                        gtag('config', '".esc_html($settings['googleAnalytics'])."')";
        
            echo wp_get_script_tag( array(
                'async' => true,
                'src' => esc_url("https://www.googletagmanager.com/gtag/js?id=".$settings['googleAnalytics']).""
            ) );

            echo wp_get_inline_script_tag( $getScript );
        }
       
    }

}

